"""OAuth 2.0 WSGI server middleware providing MyProxy certificates as access tokens
"""
__author__ = "W V Engen"
__date__ = "13/11/12"
__copyright__ = "(C) 2012 FOM / Nikhef"
__license__ = "BSD - see LICENSE file in top-level directory"
__contact__ = "wvengen@nikhef.nl"
__revision__ = "$Id$"
import logging
from ConfigParser import SafeConfigParser
log = logging.getLogger(__name__)


class ResourceRegistration(object):
    """
    An entry in the resource register.
    """
    def __init__(self, name, resource_id,
                 resource_secret, authentication_data):
        self.name = name
        self.id = resource_id
        self.secret = resource_secret
        self.authentication_data = authentication_data


class ResourceRegister(object):
    """
    Resource reqister read from a configuration file
    """
    register = {}
    def __init__(self, config_file=None):
        if config_file:
            config = SafeConfigParser()
            config.read(config_file)
            resource_keys = config.get('resource_register', 'resources').strip()

            if resource_keys:
                for resource_key in [k.strip() for k in resource_keys.split(',')]:
                    self._create_resource(config, resource_key, 'resource')

    def _create_resource(self, config, resource_key, prefix):
        resource_section_name = prefix + ':' + resource_key
        resource_id = config.get(resource_section_name, 'id')
        resource_secret = None

        if config.has_option(resource_section_name, 'secret'):
            resource_secret = config.get(resource_section_name, 'secret')

        resource_authentication_data = None
        if config.has_option(resource_section_name, 'authentication_data'):
            resource_authentication_data = config.get(resource_section_name, 'authentication_data')

        resource_registration = ResourceRegistration(
            config.get(resource_section_name, 'name'),
            resource_id,
            resource_secret,
            resource_authentication_data)
        self.register[resource_id] = resource_registration

    def is_registered_resource(self, resource_id):
        """Determines if a resource ID is in the resource register.
        """
        if resource_id not in self.register:
            return ('Resource of id "%s" is not registered.' % resource_id)

        return None




"""OAuth 2.0 WSGI server middleware providing MyProxy certificates as access tokens
"""
__author__ = "R B Wilkinson"
__date__ = "12/12/11"
__copyright__ = "(C) 2011 Science and Technology Facilities Council"
__license__ = "BSD - see LICENSE file in top-level directory"
__contact__ = "Philip.Kershaw@stfc.ac.uk"
__revision__ = "$Id$"

from ndg.oauth.server.lib.authenticate.authenticator_interface import AuthenticatorInterface
from ndg.oauth.server.lib.oauth.oauth_exception import OauthException


class CertificateAuthenticator(AuthenticatorInterface):
    CERT_DN_ENVIRON_KEY = 'SSL_CLIENT_S_DN'
    """
    Client authenticator implementation that checks for a SSL certificate DN
    in the environ and compares this with that registered for the client/resource/....
    SSL certificate authentication must be configured, e.g., in an Apache server
    hosting the application.
    """

    def __init__(self, typ, register):
        AuthenticatorInterface.__init__(self, typ)
        self._register = register

    def authenticate(self, request):
        """
        Checks for an SSL certificate distinguished name in the environ and if
        found, returns it.
        @type request: webob.Request
        @param request: HTTP request object

        @rtype: str
        @return: ID of authenticated client/resource, or None if authentication
        is not to be performed.
        
        Raise OauthException if authentication fails.
        """
        dn = request.environ.get(self.CERT_DN_ENVIRON_KEY)
        if not dn:
            raise OauthException('invalid_%s' % self.typ, 'No certificate DN found.')

        for authorization in self._register.register.itervalues():
            if authorization.authentication_data == dn:
                return authorization.id

        raise OauthException('invalid_%s' % self.typ,
			     'Certificate DN does not match that for any registered %s: %s' % (self.typ, dn))




'''
Created on 17 Apr 2015

@author: philipkershaw
'''
from zope.interface import directlyProvides

from repoze.who._compat import REQUEST_METHOD
from repoze.who.interfaces import IRequestClassifier


def oauth2basicauth_request_classifier(environ):
    """Return one of the following classifiers:

    'oauth2basicauth':  the request comes from OAuth 2.0 client
    requesting an access token.
    """
    request_method = REQUEST_METHOD(environ)

    if (request_method == 'POST' and 
        environ.get('PATH_INFO', '').startswith('/oauth/access_token')):
        return 'oauth2basicauth'
    else:
        return 'browser'

directlyProvides(oauth2basicauth_request_classifier, IRequestClassifier)


"""OAuth 2.0 WSGI server middleware providing MyProxy certificates as access tokens
"""
__author__ = "R B Wilkinson"
__date__ = "12/12/11"
__copyright__ = "(C) 2011 Science and Technology Facilities Council"
__license__ = "BSD - see LICENSE file in top-level directory"
__contact__ = "Philip.Kershaw@stfc.ac.uk"
__revision__ = "$Id$"

import base64
import logging

from ndg.oauth.server.lib.access_token.access_token_interface import AccessTokenInterface
from ndg.oauth.server.lib.register.access_token import AccessToken

log = logging.getLogger(__name__)

class MyProxyCertTokenGenerator(AccessTokenInterface):
    """Access token generator that returns MyProxy certificates as tokens.
    """
    def __init__(self, lifetime, token_type, **kw):
        """
        @type lifetime: int
        @param lifetime: lifetimes of generated tokens in seconds

        @type token_type: str
        @param token_type: token type name

        @type kw:dict
        @param kw: additional keywords
        """
        self.lifetime = lifetime
        self.token_type = token_type
        self.certificate_request_parameter = kw.get('certificate_request_parameter')
        self.myproxy_client_env_key = kw.get('myproxy_client_env_key',
            'myproxy.server.wsgi.middleware.MyProxyClientMiddleware.myProxyClient')
        self.myproxy_global_password = kw.get('myproxy_global_password')
        self.user_identifier_grant_data_key = kw.get('user_identifier_grant_data_key')

    def get_access_token(self, token_request, grant, request):
        """
        Gets an access token using MyProxyClient.
        @type token_request: ndg.oauth.server.lib.access_token.AccessTokenRequest
        @param token_request: access token request

        @type grant: ndg.oauth.server.lib.register.authorization_grant.AuthorizationGrant
        @param grant: authorization grant

        @type request: webob.Request
        @param request: HTTP request object

        @rtype: ndg.oauth.server.lib.register.access_token.AccessToken
        @return: access token or None if an error occurs
        """
        myproxyclient = request.environ.get(self.myproxy_client_env_key)
        if myproxyclient is None:
            log.error('MyProxy client not found in environ')
            return None

        cert_req_enc = request.POST.get(self.certificate_request_parameter)
        if cert_req_enc is None:
            log.error('Certificate request not found in POST parameters')
            return None
        cert_req = base64.b64decode(cert_req_enc)

        # Get the user identification as set by an authentication filter.
        myproxy_id = grant.additional_data.get(
                                            self.user_identifier_grant_data_key)
        if not myproxy_id:
            log.error('User identifier not stored with grant')
            return None

        # Attempt to obtain a certificate from MyProxy.
        try:
            creds = myproxyclient.logon(myproxy_id, 
                                        self.myproxy_global_password, 
                                        certReq=cert_req)
        except Exception, exc:
            log.error('MyProxy logon failed: %s', exc.__str__())
            return None

        token_id = creds[0]
        return AccessToken(token_id, token_request, grant, self.token_type, 
                           self.lifetime)



"""OAuth 2.0 WSGI server middleware providing MyProxy certificates as access tokens
"""
__author__ = "R B Wilkinson"
__date__ = "12/12/11"
__copyright__ = "(C) 2011 Science and Technology Facilities Council"
__license__ = "BSD - see LICENSE file in top-level directory"
__contact__ = "Philip.Kershaw@stfc.ac.uk"
__revision__ = "$Id$"

import logging
import uuid

from ndg.oauth.server.lib.authorize.authorizer_interface import AuthorizerInterface
from ndg.oauth.server.lib.oauth.oauth_exception import OauthException
from ndg.oauth.server.lib.register.authorization_grant import AuthorizationGrant

log = logging.getLogger(__name__)

class AuthorizerStoringIdentifier(AuthorizerInterface):
    """
    Authorizer interface that stores the user identifier as additional grant
    data so that it can be used in access token generation.
    """
    def __init__(self, lifetime, **kw):
        """
        @type lifetime: int
        @param lifetime: lifetimes of generated tokens in seconds

        @type kw:dict
        @param kw: additional keywords
            user_identifier_env_key: key for user identifier value in environ
            user_identifier_grant_data_key: key in additional grant data for
                user identifier value
        """
        self.lifetime = lifetime
        self.user_identifier_env_key = kw.get('user_identifier_env_key')
        self.user_identifier_grant_data_key = kw.get(
                                            'user_identifier_grant_data_key')

    def generate_authorization_grant(self, auth_request, request):
        """Generates an authorization grant.
        @type auth_request: ndg.oauth.server.lib.oauth.authorize.AuthorizeRequest
        @param auth_request: authorization request

        @type request: webob.Request
        @param request: HTTP request object

        @rtype: tuple (
            ndg.oauth.server.lib.register.authorization_grant.AuthorizationGrant
            str
        )
        @return: tuple (
            authorization grant
            authorization code
        )
        """
        user_identifier = request.environ.get(self.user_identifier_env_key)
        if not user_identifier:
            log.error('Could not find user identifier key "%s" in environ', 
                      self.user_identifier_env_key)
            raise OauthException('server_error', 
                                 'Authorization grant could not be created')

        code = uuid.uuid4().hex
        additional_data = {self.user_identifier_grant_data_key: user_identifier}
        grant = AuthorizationGrant(code, auth_request, self.lifetime,
                                   additional_data=additional_data)
        return grant, code



"""OAuth 2.0 WSGI server middleware implements Authorisation and Resource 
server functionality.  It includes support for basic bearer tokens and also 
a customisation - X.509 certificates as access tokens - for use with an SLCS
(Short-lived Credential Service)
"""
__author__ = "R B Wilkinson"
__date__ = "12/12/11"
__copyright__ = "(C) 2011 Science and Technology Facilities Council"
__license__ = "BSD - see LICENSE file in top-level directory"
__contact__ = "Philip.Kershaw@stfc.ac.uk"
__revision__ = "$Id$"

import httplib
import logging
import urllib
import urlparse

from webob import Request

from ndg.oauth.server.lib.access_token.bearer_token_generator import \
    BearerTokenGenerator
from ndg.oauth.server.lib.authenticate.certificate_authenticator \
    import CertificateAuthenticator
from ndg.oauth.server.lib.authenticate.noop_authenticator import \
    NoopAuthenticator
from ndg.oauth.server.lib.authenticate.password_authenticator \
    import PasswordAuthenticator
from ndg.oauth.server.lib.authorization_server import AuthorizationServer
from ndg.oauth.server.lib.authorize.authorizer_storing_identifier import \
    AuthorizerStoringIdentifier
from ndg.oauth.server.lib.register.client import ClientRegister
from ndg.oauth.server.lib.register.resource import ResourceRegister

log = logging.getLogger(__name__)


class Oauth2ServerMiddleware(object):
    """
    WSGI OAuth 2.0 server implements Authorisation and Resource server functions
    
    Requires:
    o Beaker session
    o An authentication provider that sets the user's ID in the environ, e.g.,
      repose.who
    o MyProxyClientMiddleware (ONLY required for X.509 cert based token
      generation used with SLCS config)
    o Middleware to set user's decisions for authorization of OAuth clients in
      the environ, e.g., ndg.oauth.server.wsgi.authorization_filter.
    """
    PARAM_PREFIX = 'oauth2server.'
    CERT_DN_ENVIRON_KEY = 'SSL_CLIENT_S_DN'
    
    # Configuration options
    ACCESS_TOKEN_LIFETIME_OPTION = 'access_token_lifetime'
    ACCESS_TOKEN_TYPE_OPTION = 'access_token_type'
    AUTHORIZATION_GRANT_LIFETIME_OPTION = 'authorization_grant_lifetime'
    BASE_URL_PATH_OPTION = 'base_url_path'
    CERTIFICATE_REQUEST_PARAMETER_OPTION = 'certificate_request_parameter'
    CLIENT_AUTHENTICATION_METHOD_OPTION = 'client_authentication_method'
    CLIENT_AUTHORIZATION_URL_OPTION = 'client_authorization_url'
    CLIENT_AUTHORIZATIONS_KEY_OPTION = 'client_authorizations_key'
    CLIENT_REGISTER_OPTION = 'client_register'
    RESOURCE_AUTHENTICATION_METHOD_OPTION = 'resource_authentication_method'
    RESOURCE_REGISTER_OPTION = 'resource_register'
    MYPROXY_CLIENT_KEY_OPTION = 'myproxy_client_key'
    MYPROXY_GLOBAL_PASSWORD_OPTION = 'myproxy_global_password'
    USER_IDENTIFIER_KEY_OPTION = 'user_identifier_key'
    USER_IDENTIFIER_GRANT_DATA_KEY = 'user_identifier'

    AUTHORISATION_SERVER_ENVIRON_KEYNAME = \
                                        'ndg.oauth.server.authorisation.server'
    
    # Configuration option defaults
    PROPERTY_DEFAULTS = {
        ACCESS_TOKEN_LIFETIME_OPTION: 86400,
        ACCESS_TOKEN_TYPE_OPTION: 'bearer',
        AUTHORIZATION_GRANT_LIFETIME_OPTION: 600,
        BASE_URL_PATH_OPTION: '',
        CERTIFICATE_REQUEST_PARAMETER_OPTION: 'certificate_request',
        CLIENT_AUTHENTICATION_METHOD_OPTION: 'certificate',
        CLIENT_AUTHORIZATION_URL_OPTION: '/client_authorization/authorize',
        CLIENT_AUTHORIZATIONS_KEY_OPTION: 'client_authorizations',
        RESOURCE_AUTHENTICATION_METHOD_OPTION: 'none',
        MYPROXY_CLIENT_KEY_OPTION: \
        'myproxy.server.wsgi.middleware.MyProxyClientMiddleware.myProxyClient',
        USER_IDENTIFIER_KEY_OPTION: 'REMOTE_USER'
    }
    method = {
        '/access_token': 'access_token',
        '/authorize': 'authorize',
        '/check_token': 'check_token',
        '/request_certificate': 'request_certificate'
    }

    def __init__(self, app, app_conf, prefix=PARAM_PREFIX, **local_conf):
        """
        Sets up the server depending on the configuration.

        @type app: WSGI application
        @param app: wrapped application/middleware

        @type app_conf: dict
        @param app_conf: application configuration settings - ignored - this
        method includes this arg to fit Paste middleware / app function 
        signature

        @@type prefix: str
        @param prefix: optional prefix for parameter names included in the 
        local_conf dict - enables these parameters to be filtered from others
        which don't apply to this middleware

        @type local_conf: dict
        @param local_conf: attribute settings to apply
        """
        self._app = app
        conf = self._set_configuration(prefix, local_conf)

        if self.access_token_type == 'bearer':
            # Simple bearer token configuration.
            access_token_generator = BearerTokenGenerator(
                                        self.access_token_lifetime_seconds, 
                                        self.access_token_type)
        else:
            raise ValueError("Invalid configuration value %s for %s" %
                             (self.access_token_type,
                              self.ACCESS_TOKEN_TYPE_OPTION))
            
        # Store user identifier with grant - this isn't needed for the OAuth
        # protocol but is needed to return certificates using MyProxy.
        authorizer = AuthorizerStoringIdentifier(
            self.authorization_grant_lifetime_seconds,
            user_identifier_env_key=self.user_identifier_env_key,
            user_identifier_grant_data_key=self.USER_IDENTIFIER_GRANT_DATA_KEY)

        # Determine client authentication type. A 'none' options is allowed so
        # that development/testing can be performed without running on Apache.
        client_register = ClientRegister(self.client_register_file)
        client_authenticator = self._get_authenticator(
            self.client_authentication_method, client_register,
            'client', self.CLIENT_AUTHENTICATION_METHOD_OPTION)

        # same for resource authentication type.
        resource_register = ResourceRegister(self.resource_register_file)
        resource_authenticator = self._get_authenticator(
            self.resource_authentication_method, resource_register,
            'resource', self.RESOURCE_AUTHENTICATION_METHOD_OPTION)

        self._authorizationServer = AuthorizationServer(
            client_register, authorizer, client_authenticator,
            resource_register, resource_authenticator,
            access_token_generator, conf)

    def _get_authenticator(self, name, register, typ, option_name):
        """Returns new authenticator by name"""
        if name == 'certificate':
            return CertificateAuthenticator(typ, register)
            
        elif name == 'password':
            return PasswordAuthenticator(typ, register)
            
        elif name == 'none':
            return NoopAuthenticator(typ)
            
        else:
            raise ValueError("Invalid configuration value %s for %s" %
                             (name, option_name))

    def __call__(self, environ, start_response):
        """
        @type environ: dict
        @param environ: WSGI environment

        @type start_response: WSGI start response function
        @param start_response: start response function

        @rtype: iterable
        @return: WSGI response
        """
        log.debug("Oauth2ServerMiddleware.__call__ ...")
        req = Request(environ)
        log.debug("Request path_info: %s", req.path_info)

        # Set Authorisation Server as key in environ for access by downstream
        # middleware or app.  For example, a resource server can use it to 
        # check access tokens presented to it from clients against ones issued
        # by the authorisation server
        environ[self.__class__.AUTHORISATION_SERVER_ENVIRON_KEYNAME
                ] = self._authorizationServer
                
        # Determine what operation the URL specifies.
        actionPath = None
        if req.path_info.startswith(self.base_path):
            actionPath = req.path_info[len(self.base_path):]

        methodName = self.__class__.method.get(actionPath, '')
        if methodName:
            log.debug("Method: %s" % methodName)
            action = getattr(self, methodName)
            return action(req, start_response)

        elif self._app is not None:
            log.debug("Delegating to lower filter/application.")
            return self._app(environ, start_response)

        else:
            response = "OAuth 2.0 Server - Invalid URL"
            start_response(self._get_http_status_string(httplib.NOT_FOUND),
                           [('Content-type', 'text/plain'),
                            ('Content-length', str(len(response)))
                            ])
            return [response]

    def authorize(self, req, start_response):
        """Handles OAuth 2 authorize request.
        @type req: webob.Request
        @param req: HTTP request object

        @type start_response: WSGI start response function
        @param start_response: start response function

        @rtype: iterable
        @return: WSGI response
        """
        # Stop immediately if the client is not registered.
        (error, error_description
                        ) = self._authorizationServer.is_registered_client(req)
        if error:
            log.debug("Error checking if client registered: %s - %s", error,
                      error_description)
            return self._error_response(error, error_description,
                                        start_response)

        # User authentication is required before authorization can proceed.
        user = req.environ.get(self.user_identifier_env_key)
        if not user:
            log.debug("%s not in environ - authentication required",
                      self.user_identifier_env_key)
            start_response(self._get_http_status_string(httplib.UNAUTHORIZED), 
                           [])
            return []

        # User authorization for the client is also required.
        client_authorized, authz_uri = self._check_client_authorization(user, 
                                                                        req)
        if authz_uri:
            log.debug("Redirecting to %s", authz_uri)
            return self._redirect(authz_uri, start_response)

        if not client_authorized:
            log.debug("User has declined authorization for client.")

        # Request authorization grant or for Implicit Grant flow
        (redirect_uri, 
         error, 
         error_description) = self._authorizationServer.authorize(
                                                            req, 
                                                            client_authorized)
        if error:
            return self._error_response(error, 
                                        error_description, 
                                        start_response)
        else:
            return self._redirect(redirect_uri, start_response)

    def _error_response(self, error, error_description, start_response):
        """Returns and error response.
        """
        response = ("%s: %s" % (error, error_description)).encode('ascii',
                                                                  'ignore')
        log.error("Returning error: %s - %s", error, error_description)
        start_response(self._get_http_status_string(httplib.BAD_REQUEST),
                       [('Content-type', 'text/plain'),
                        ('Content-length', str(len(response)))
                        ])
        return[response]

    def _check_client_authorization(self, user, req):
        """
        Gets the URL to which to redirect for the user to authorize the client.
        Returns: URL or None if already authorized
        @type user: str
        @param user: identifier of user/resource owner

        @type req: webob.Request
        @param req: HTTP request object
        """
        client_authorizations = req.environ.get(
                                            self.client_authorizations_env_key)
        client_id = req.params.get('client_id')
        scope = req.params.get('scope')
        if not client_authorizations:
            client_authorized = None
            log.debug("Client authorization register not found in environ "
                      "(key %s).", self.client_authorizations_env_key)
        else:
            client_authorized = \
                client_authorizations.is_client_authorized_by_user(user, 
                                                                   client_id, 
                                                                   scope)
        if client_authorized is None:
            log.debug("Client not authorized by user (client_id: %s  scope: %s"
                      "  user: %s).", client_id, scope, user)
            
            url_params = {'client_id': client_id,
                          'scope': scope,
                          'user': user,
                          'original_url': req.url}
            url = ("%s?%s" % (self._make_client_authorization_url(req),
                              urllib.urlencode(url_params)))
            return (None, url)
        
        return client_authorized, None

    def _make_client_authorization_url(self, req):
        parts = urlparse.urlparse(self.client_authorization_url)
        if (parts.scheme and parts.netloc):
            # Absolute URL:
            return self.client_authorization_url
        else:
            # Relative - append to application URL:
            return '/'.join([req.application_url.rstrip('/'),
                             self.client_authorization_url.lstrip('/')])


    def access_token(self, req, start_response):
        """Handles OAuth 2 access token request.
        @type req: webob.Request
        @param req: HTTP request object

        @type start_response: 
        @param start_response: WSGI start response function

        @rtype: iterable
        @return: WSGI response
        """
        log.debug("access_token called")
        (response, 
         error_status, 
         error_description) = self._authorizationServer.access_token(req)
        if response is None:
            response = ''
        headers = [
            ('Content-Type', 'application/json; charset=UTF-8'),
            ('Cache-Control', 'no-store'),
            ('Content-length', str(len(response))),
            ('Pragma', 'no-store')
        ]
        status_str = self._get_http_status_string(
                                error_status if error_status else httplib.OK)
        if error_status:
            log.debug("Error obtaining access token: %s - %s", status_str,
                      error_description)

        start_response(status_str, headers)
        return [response]

    def check_token(self, req, start_response):
        """
        Service to validate bearer tokens. It would be called from a resource
        service that trusts this authorization service.
        @type req: webob.Request
        @param req: HTTP request object

        @type start_response: 
        @param start_response: WSGI start response function

        @rtype: iterable
        @return: WSGI response
        """
        response, error_status = self._authorizationServer.check_token(req)[0:2]
        if response is None:
            response = ''
        headers = [
            ('Content-Type', 'application/json; charset=UTF-8'),
            ('Cache-Control', 'no-store'),
            ('Content-length', str(len(response))),
            ('Pragma', 'no-store')
        ]
        status_str = self._get_http_status_string(
                                error_status if error_status else httplib.OK)

        start_response(status_str, headers)
        return [response]

    @staticmethod
    def _get_http_status_string(status):
        return ("%d %s" % (status, httplib.responses[status]))

    def _redirect(self, url, start_response):
        """Initiates a redirect to a specified URL.
        @type param: str
        @param url: URL

        @type start_response: 
        @param start_response: WSGI start response function

        @rtype: iterable
        @return: WSGI response
        """
        log.debug("Redirecting to %s", url)
        start_response(self._get_http_status_string(httplib.FOUND),
               [('Location', url.encode('ascii', 'ignore'))])
        return []

    def _set_configuration(self, prefix, local_conf):
        """Sets the configuration values.

        @type prefix: str
        @param prefix: optional prefix for parameter names included in the
        local_conf dict - enables these parameters to be filtered from others
        which don't apply to this middleware

        @type local_conf: dict
        @param local_conf: attribute settings to apply
        """
        conf = {}
        plen = len(prefix)
        for k, v in local_conf.iteritems():
            if k.startswith(prefix):
                conf[k[plen:]] = v
        cls = self.__class__
        
        self.base_path = cls._get_config_option(conf, cls.BASE_URL_PATH_OPTION)
        self.authorization_grant_lifetime_seconds = cls._get_config_option(
                                conf, cls.AUTHORIZATION_GRANT_LIFETIME_OPTION)
        self.access_token_lifetime_seconds = cls._get_config_option(
                                conf, cls.ACCESS_TOKEN_LIFETIME_OPTION)
        self.access_token_type = cls._get_config_option(
                                conf, cls.ACCESS_TOKEN_TYPE_OPTION)
        self.certificate_request_parameter = cls._get_config_option(
                                conf, cls.CERTIFICATE_REQUEST_PARAMETER_OPTION)
        self.client_authorization_url  = cls._get_config_option(
                                conf, cls.CLIENT_AUTHORIZATION_URL_OPTION)
        self.client_authentication_method  = cls._get_config_option(
                                conf, cls.CLIENT_AUTHENTICATION_METHOD_OPTION)
        self.client_authorizations_env_key = cls._get_config_option(
                                conf, cls.CLIENT_AUTHORIZATIONS_KEY_OPTION)
        self.client_register_file = cls._get_config_option(
                                conf, cls.CLIENT_REGISTER_OPTION)
        self.resource_authentication_method = cls._get_config_option(
                                conf, cls.RESOURCE_AUTHENTICATION_METHOD_OPTION)
        self.resource_register_file = cls._get_config_option(
                                conf, cls.RESOURCE_REGISTER_OPTION)
        self.myproxy_client_env_key = cls._get_config_option(
                                conf, cls.MYPROXY_CLIENT_KEY_OPTION)
        self.myproxy_global_password = cls._get_config_option(
                                conf, cls.MYPROXY_GLOBAL_PASSWORD_OPTION)
        self.user_identifier_env_key = cls._get_config_option(
                                conf, cls.USER_IDENTIFIER_KEY_OPTION)
        
        # Return any options that start with the prefix but haven't been read
        # above.
        return conf

    @classmethod
    def _get_config_option(cls, conf, key):
        value = conf.pop(key, cls.PROPERTY_DEFAULTS.get(key, None))
        log.debug("Oauth2ServerMiddleware configuration %s=%s", key, value)
        return value

    @classmethod
    def filter_app_factory(cls, app, app_conf, **local_conf):
        return cls(app, app_conf, **local_conf)

    @classmethod
    def app_factory(cls, app_conf, **local_conf):
        return cls(None, app_conf, **local_conf)




"""OAuth 2.0 WSGI server middleware providing MyProxy certificates as access tokens
"""
__author__ = "R B Wilkinson"
__date__ = "12/12/11"
__copyright__ = "(C) 2011 Science and Technology Facilities Council"
__license__ = "BSD - see LICENSE file in top-level directory"
__contact__ = "Philip.Kershaw@stfc.ac.uk"
__revision__ = "$Id$"

from datetime import datetime, timedelta
import logging
import uuid

from ndg.oauth.server.lib.register.register_base import RegisterBase
import ndg.oauth.server.lib.register.scopeutil as scopeutil

log = logging.getLogger(__name__)

class AccessToken(object):
    """
    Access token as stored in the reqister
    """
    def __init__(self, token_type, lifetime):
        self.token_id = uuid.uuid4().hex
        self.token_type = token_type
        self.grant = None
        self.scope = None
        self.timestamp = datetime.now()
        self.lifetime = lifetime
        self.expires = self.timestamp + timedelta(days=0, seconds=lifetime)
        self.valid = True
              
    @classmethod  
    def from_token_request(cls, token_type, grant, lifetime):
        '''Create an instance from a token request.  This applies to the 
        Authorization Code Grant flow
        '''
        obj = cls(token_type, lifetime)
        
        obj.token_type = token_type
        obj.grant = grant        
        obj.scope = scopeutil.scopeStringToList(grant.scope_str)
        
        return obj

    @classmethod
    def from_authz_request(cls, token_type, authz_request, lifetime):
        '''Create an instance from an authorisation request.  This applies to 
        the Implicit Grant flow
        '''
        obj = cls(token_type, lifetime)

        obj.token_type = token_type
        obj.scope = authz_request.scope
        
        return obj

    @classmethod
    def create(cls, token_type, _arg, lifetime):
        if hasattr(_arg, 'scope_str'):
            # Assume a 'grant' object - authorisation code grant flow
            method_name = 'from_token_request'
            
        else:
            # Implicit grant flow
            method_name = 'from_authz_request'
            
        method = getattr(cls, method_name)
        
        return method(token_type, _arg, lifetime)
            

class AccessTokenRegister(RegisterBase):
    """
    Access token reqister that holds access tokens as determined by the cache
    options
    """
    CACHE_NAME = 'accesstokenregister'

    def __init__(self, config, prefix='cache'):
        cache_opts = self.parse_config(prefix, self.CACHE_NAME, config)
        super(AccessTokenRegister, self).__init__('AccessTokenRegister', cache_opts)

    def add_token(self, token):
        """Adds a token to the register.
        @type token: AccessToken
        @param token: access token
        """
        if self.has_key(token.token_id):
            # Internal error
            log.error("Repeated attempt to add token of ID: %s", token.token_id)
            return False

        self.set_value(token.token_id, token)
        log.debug("Added token of ID: %s", token.token_id)
        return True

    def get_token(self, token_id, scope):
        """Retrieves a registered token by token ID and required scope.
        @type token_id: basestring
        @param token_id: token ID
        @type scope: basestring
        @param scope: required scopes as space separated string
        """
        try:
            token = self.get_value(token_id)
        except KeyError:
            log.debug("Request for token of ID that is not registered: %s",
                      token_id)
            return None, 'invalid_token'

        if not token.valid:
            log.debug("Request for invalid token of ID: %s", token_id)
            return None, 'invalid_token'
        
        if token.expires <= datetime.utcnow():
            log.debug("Request for expired token of ID: %s", token_id)
            return None, 'invalid_token'
                    
        # Check scope
        if not scopeutil.isScopeGranted(token.scope,
                                        scopeutil.scopeStringToList(scope)):
            log.debug("Request for token of ID: %s - token was not granted "
                      "scope %s", token_id, scope)
            return None, 'insufficient_scope'
        
        return token, None



"""OAuth 2.0 WSGI server middleware - utilities for handling scope strings and
lists
"""
__author__ = "R B Wilkinson"
__date__ = "28/03/12"
__copyright__ = "(C) 2012 Science and Technology Facilities Council"
__license__ = "BSD - see LICENSE file in top-level directory"
__contact__ = "Philip.Kershaw@stfc.ac.uk"
__revision__ = "$Id$"

import logging
import urllib

log = logging.getLogger(__name__)

def scopeStringToList(scope_str):
    """Converts a scope string to a list. The strings are space separated and
    must not contain '"' or '\' so allow them to be URL encoded.
    @type scope_str: basestring
    @param scope_str: space separated list of scopes
    @rtype: list of basestring
    @return: list of scopes
    """
    result = []
    if scope_str:
        for s in scope_str.split():
            result.append(urllib.unquote_plus(s))
    log.debug("Converted scope string %s to %r", scope_str, result)
    return result

def isScopeGranted(granted_scope, requested_scope):
    """Determines whether all scopes requested have been granted.
    @type granted_scope: list of basestring
    @param granted_scope: list of granted scopes
    @type requested_scope: list of basestring
    @param requested_scope: list of requested scopes
    @rtype: bool
    @return: True if all requested scopes are in the list of granted scopes,
    or if no scopes are requested, otherwise False
    """
    result = True
    for s in requested_scope:
        if s not in granted_scope:
            result = False
    log.debug(
        "Checking for requested scopes %r in granted scopes %r - result: %r",
        requested_scope, granted_scope, result)
    return result



#!/usr/bin/env python
"""NDG OAuth test harness for bearer token example

"""
__author__ = "P J Kershaw"
__date__ = "20/11/08"
__copyright__ = "(C) 2012 Science and Technology Facilities Council"
__contact__ = "Philip.Kershaw@stfc.ac.uk"
__revision__ = "$Id:$"  
from os import path
                                     
from ndg.oauth.server.examples.utils import serve_app
                                      
INI_FILENAME = 'bearer_tok_server_app.ini'

# To start run 
# $ paster serve services.ini or run this file as a script, see
# $ ./securityservicesapp.py -h
if __name__ == '__main__':    
    config_filepath = path.join(path.dirname(path.abspath(__file__)), 
                                INI_FILENAME) 
    serve_app(INI_FILENAME)


"""OAuth 2.0 WSGI server middleware providing MyProxy certificates in response
to resource requests
"""
__author__ = "R B Wilkinson"
__date__ = "13/03/12"
__copyright__ = "(C) 2011 Science and Technology Facilities Council"
__license__ = "BSD - see LICENSE file in top-level directory"
__contact__ = "Philip.Kershaw@stfc.ac.uk"
__revision__ = "$Id$"

import base64
import logging

log = logging.getLogger(__name__)


class MyproxyCertRequest(object):
    def __init__(self, **kw):
        """
        @type kw: dict
        @param kw: additional keywords
        """
        self.certificate_request_parameter = kw.get(
                                                'certificate_request_parameter')
        self.myproxy_client_env_key = kw.get('myproxy_client_env_key',
            'myproxy.server.wsgi.middleware.MyProxyClientMiddleware.myProxyClient')
        self.myproxy_global_password = kw.get('myproxy_global_password')
        self.user_identifier_grant_data_key = kw.get(
                                            'user_identifier_grant_data_key')

    def get_resource(self, token, request):
        """ Creates response to a resource request - takes a certificate request
        from the request parameters and obtains a certificate from a MyProxy
        server.
        @type token: basestring
        @param token: access token
        @type request: webob.Request
        @param request: request object
        @rtype: basestring
        @returns: certificate
        """
        grant = token.grant

        myproxyclient = request.environ.get(self.myproxy_client_env_key)
        if myproxyclient is None:
            log.error('MyProxy client not found in environ')
            return None

        cert_req_enc = request.POST.get(self.certificate_request_parameter)
        if cert_req_enc is None:
            log.error('Certificate request not found in POST parameters')
            return None
        cert_req = base64.b64decode(cert_req_enc)

        # Get the user identification as set by an authentication filter.
        myproxy_id = grant.additional_data.get(
                                            self.user_identifier_grant_data_key)
        if not myproxy_id:
            log.error('User identifier not stored with grant')
            return None

        # Attempt to obtain a certificate from MyProxy.
        try:
            creds = myproxyclient.logon(myproxy_id,
                                        self.myproxy_global_password,
                                        certReq=cert_req)
        except Exception, exc:
            log.error('MyProxy logon failed: %s', exc.__str__())
            return None

        return creds[0]



"""OAuth 2.0 WSGI server middleware providing MyProxy certificates as access tokens
"""
__author__ = "Philip Kershaw"
__date__ = "19/10/12"
__copyright__ = "(C) 2012 Science and Technology Facilities Council"
__license__ = "BSD - see LICENSE file in top-level directory"
__contact__ = "Philip.Kershaw@stfc.ac.uk"
__revision__ = "$Id$"
import logging
import httplib
import re

import json
from webob import Request

from ndg.oauth.server.wsgi.oauth2_server import Oauth2ServerMiddleware
from ndg.oauth.server.lib.authorization_server import AuthorizationServer

log = logging.getLogger(__name__)
is_iterable = lambda obj: getattr(obj, '__iter__', False) 


class Oauth2ResourceServerMiddlewareError(Exception):
    '''Base exception class for OAuth 2.0 Resource Server errors'''
    

class Oauth2ResourceServerMiddlewareConfigError(
                                        Oauth2ResourceServerMiddlewareError):
    '''Exception class for OAuth 2.0 Resource Server configuration errors'''
    

class Oauth2ResourceServerMiddleware(object):
    '''OAuth 2.0 Resource Server implemented as a WSGI filter.  This filter 
    fronts a given applications resources in order to secure them.  Requests
    are intercepted and validated for an acceptable Access Token
    '''
    CERT_DN_ENVIRON_KEY = 'SSL_CLIENT_S_DN'
    DEFAULT_PARAM_PREFIX = 'oauth2.resource_server.'
    
    MATCH_SCOPE_TO_CLIENT_DN_OPTNAME = 'match_scope_to_client_dn'
    RESOURCE_URIPATHS_OPTNAME = 'resource_uripaths'
    
    CLAIMED_USER_ID_ENVIRON_KEY_OPTNAME = 'claimed_userid_environ_key'
    DEFAULT_CLAIMED_USER_ID_ENVIRON_KEYNAME = \
        'oauth.resource_server.claimed_userid'
    
    AUTHORISATION_SERVER_ENVIRON_KEYNAME = \
        Oauth2ServerMiddleware.AUTHORISATION_SERVER_ENVIRON_KEYNAME
        
    __slots__ = (
        '_app',
        '__authorization_server',
        'claimed_userid_environ_key',
        '__resource_uripaths',
        '__required_scope'
    )
    def __init__(self, app):
        self._app = app
        self.__authorization_server = None
        self.claimed_userid_environ_key = \
            self.__class__.DEFAULT_CLAIMED_USER_ID_ENVIRON_KEYNAME
        self.__resource_uripaths = []
        
        # Scope for this resource - multiple space delimited scope values may
        # be set
        self.__required_scope = None
        
    @classmethod
    def filter_app_factory(cls, app, global_conf, prefix=DEFAULT_PARAM_PREFIX,
                           **app_conf):
        obj = cls(app)
        obj.parse_keywords(**app_conf)
        return obj
        
    def parse_keywords(self, prefix=DEFAULT_PARAM_PREFIX, **conf):
        '''Set attributes from keywords passed
        
        @type prefix: basestring
        @param prefix: if a prefix is given, only update self from kw items 
        where keyword starts with this prefix
        @type conf: dict
        @param conf: items corresponding to instance variables to 
        update.  Keyword names must match their equivalent instance 
        variable names.  However, they may prefixed with <prefix>
        '''
        if prefix is None:
            prefix_ = ''
        else:
            prefix_ = prefix
            
        prefix_len = len(prefix_)
            
        for optname, val in conf.items():
            if optname.startswith(prefix_):
                setattr(self, optname[prefix_len:], val)
        
    @property
    def resource_uripaths(self):
        return self.__resource_uripaths
    
    @resource_uripaths.setter
    def resource_uripaths(self, val):
        if isinstance(val, basestring):
            self.__resource_uripaths = [re.compile(path) for path in val.split()]
            
        elif is_iterable(val):
            self.__resource_uripaths = [re.compile(path) for path in val]
        else:
            raise TypeError('Expecting single string or space-separated URI '
                            'paths or an iterable; got %r instead' % type(val))
     
    @property
    def authorization_server(self):
        return self.__authorization_server
    
    @authorization_server.setter
    def authorization_server(self, val):
        if not isinstance(val, AuthorizationServer):
            raise TypeError('Expecting %r type for "authorization_server" '
                            'attribute; got %r instead' % (AuthorizationServer, 
                                                           type(val)))
        self.__authorization_server = val
     
    @property
    def required_scope(self):
        return self.__required_scope
    
    @required_scope.setter
    def required_scope(self, val):
        if not isinstance(val, basestring):
            raise TypeError('Expecting string type for "required_scope" '
                            'attribute; got %r instead' % type(val))
        self.__required_scope = val 
                       
    def __call__(self, environ, start_response):
        '''Apply validation of access token for configured resource paths
        
        @type environ: dict
        @param environ: WSGI environment variables dictionary
        @type start_response: function
        @param start_response: standard WSGI start response function
        '''
        request = Request(environ)
        
        self.authorization_server = environ.get(
                            self.__class__.AUTHORISATION_SERVER_ENVIRON_KEYNAME)
        
        if self._match_uripath(request.path_info):
            return self.request_resource(request, start_response)
        else:
            return self._app(environ, start_response)
    
    def request_resource(self, request, start_response):
        """
        Filter a resource request checking for a valid access token.  Set an
        error response if the token is invalid, otherwise pass on the request to
        the underlying app / middleware + app
        
        @type req: webob.Request
        @param req: HTTP request object

        @type start_response: 
        @param start_response: WSGI start response function

        @rtype: iterable
        @return: WSGI response
        """
        log.debug("Oauth2ResourceServerMiddleware.request_resource called for "
                  "path %r", request.path_info)

        # Check the token
        token, status, error = self.authorization_server.get_registered_token(
                                                    request, 
                                                    scope=self.required_scope)
        if not error:
            request.environ[self.claimed_userid_environ_key
                    ] = token.grant.additional_data.get('user_identifier')
                            
            return self._app(request.environ, start_response)
        else:
            status = httplib.OK
            content_dict ={}
            content_dict.setdefault('error', error)
                        
            response = json.dumps(content_dict)
            headers = [
                ('Content-Type', 'application/json; charset=UTF-8'),
                ('Cache-Control', 'no-store'),
                ('Content-length', str(len(response))),
                ('Pragma', 'no-store')
            ]
            status_str = "%d %s" % (status, httplib.responses[status])
            start_response(status_str, headers)
            return [response]
    
    def _match_uripath(self, path):
        '''Match the input request against a configured list of URI patterns
        for which this OAuth middleware should be applied
        
        @param path: URI path to match - path minus the domain name and 
        protocol specifier
        @type path: basestring
        @return: true or false for match found
        @rtype: bool
        '''
        for re_path in self.__resource_uripaths:
            if re_path.match(path):
                return True
        
        return False


"""NDG OAuth examples package

"""
__author__ = "P J Kershaw"
__date__ = "20/11/08"
__copyright__ = "(C) 2009 Science and Technology Facilities Council"
__contact__ = "Philip.Kershaw@stfc.ac.uk"
__revision__ = "$Id:$"



__author__ = "R B Wilkinson"
__date__ = "29/02/12"
__copyright__ = "(C) 2012 Science and Technology Facilities Council"
__license__ = "BSD - see LICENSE file in top-level directory"
__contact__ = "Philip.Kershaw@stfc.ac.uk"
__revision__ = "$Id$"

try:
    from setuptools import setup, find_packages
except ImportError:
    from ez_setup import use_setuptools
    use_setuptools()
    from setuptools import setup, find_packages

import os

THIS_DIR = os.path.dirname(__file__)
try:
    LONG_DESCR = open(os.path.join(THIS_DIR, 'README.md')).read()
except IOError:
    LONG_DESCR = """This is an OAuth 2.0 server library and WSGI middleware 
filter.
"""

setup(
    name =                      'ndg_oauth_server',
    version =                   '0.6.0',
    description =               'OAuth 2.0 server',
    long_description =          LONG_DESCR,
    author =                    'R. B. Wilkinson',
    maintainer =         	    'Philip Kershaw',
    maintainer_email =          'Philip.Kershaw@stfc.ac.uk',
    url =                       'https://github.com/cedadev/ndg_oauth/',
    license =                   'BSD - See LICENCE file for details',
    install_requires =[
        "PasteScript",
        "Beaker",
        "WebOb",
        "repoze.who",
        "Genshi",
    ],
    packages = find_packages(),
    package_data =      {
        'ndg.oauth.server.templates': [
            '*.html', 'public/js/*.js', 'public/layout/*.css',
            'public/layout/*.png', 'public/layout/icons/*.png'
        ],
        'ndg.oauth.server.examples': [
            'README'
        ],
        'ndg.oauth.server.examples.bearer_tok': [
            'README', 'passwd', '*.ini', 'templates/*.html', 
            'static/layout/*.css', 'static/layout/*.png'
        ],
        'ndg.oauth.server.examples.slcs': [
            'README', 'passwd', '*.ini', 'templates/*.html', 
            'static/layout/*.css', 'static/layout/*.png'
        ],
        'ndg.oauth.server.examples.shared_config': [
            'README', 'pki/*.pem', 'pki/ca/*.0'
        ]
    },
    extras_require = {
        'test-services': ['Genshi==0.6']
    },
    zip_safe = False,
    classifiers = [
        'Development Status :: 3 - Alpha',
        'Environment :: Console',
        'Environment :: Web Environment',
        'Intended Audience :: End Users/Desktop',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        'Operating System :: Microsoft :: Windows',
        'Operating System :: POSIX :: Linux',
        'Programming Language :: Python',
        'Topic :: Security',
        'Topic :: Internet',
        'Topic :: Scientific/Engineering',
        'Topic :: System :: Distributed Computing',
        'Topic :: System :: Systems Administration :: Authentication/Directory',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ],
)



"""OAuth 2.0 WSGI server middleware
"""
__author__ = "R B Wilkinson"
__date__ = "12/12/11"
__copyright__ = "(C) 2011 Science and Technology Facilities Council"
__license__ = "BSD - see LICENSE file in top-level directory"
__contact__ = "Philip.Kershaw@stfc.ac.uk"
__revision__ = "$Id$"

from datetime import datetime, timedelta
import logging

from ndg.oauth.server.lib.register.register_base import RegisterBase

log = logging.getLogger(__name__)


class AuthorizationGrant(object):
    """
    Authorization grant as stored in the reqister
    """
    def __init__(self, code, request, lifetime, scope=None, additional_data=None):
        self.code = code
        self.client_id = request.client_id
        self.redirect_uri = request.redirect_uri
        # Allow for authorized scope to be different from requested scope.
        self.scope_str = (scope if scope is not None else request.scope)
        self.additional_data = additional_data
        self.timestamp = datetime.utcnow()
        self.expires = self.timestamp + timedelta(days=0, seconds=lifetime)
        self.granted = False
        

class AuthorizationGrantRegister(RegisterBase):
    """
    Authorization grant reqister that holds access tokens as determined by the
    cache options
    """
    CACHE_NAME = 'authorizationgrantregister'

    def __init__(self, config, prefix='cache'):
        cache_opts = self.parse_config(prefix, self.CACHE_NAME, config)
        super(AuthorizationGrantRegister, self).__init__(
                                                'AuthorizationGrantRegister', 
                                                cache_opts)

    def add_grant(self, grant):
        if self.has_key(grant.code):
            # Internal error
            log.error("Repeated attempt to add grant of code: %s", grant.code)
            return False

        self.set_value(grant.code, grant)
        return True



"""OAuth 2.0 WSGI server middleware providing MyProxy certificates as access tokens
"""
__author__ = "R B Wilkinson"
__date__ = "12/12/11"
__copyright__ = "(C) 2011 Science and Technology Facilities Council"
__license__ = "BSD - see LICENSE file in top-level directory"
__contact__ = "Philip.Kershaw@stfc.ac.uk"
__revision__ = "$Id$"


class AuthorizeRequest(object):
    """
    OAuth 2.0 authorization request object
    """
    def __init__(self, response_type, client_id, redirect_uri, scope, state):
        """
        response_type
              REQUIRED.  Value MUST be set to "code".
        client_id
              REQUIRED.  The client identifier as described in Section 2.2.
        redirect_uri
              OPTIONAL, as described in Section 3.1.2.
        scope
              OPTIONAL.  The scope of the access request as described by
              Section 3.3.
        state
              RECOMMENDED.  An opaque value used by the client to maintain
              state between the request and callback.  The authorization
              server includes this value when redirecting the user-agent back
              to the client.  The parameter SHOULD be used for preventing
              cross-site request forgery as described in Section 10.12.
        """
        self.response_type = response_type
        self.client_id = client_id
        self.redirect_uri = redirect_uri
        self.scope = scope
        self.state = state


class AuthorizeResponse(object):
    """
    OAuth 2.0 authorization response object
    """
    def __init__(self, code, state):
        """
        code
              REQUIRED.  The authorization code generated by the
              authorization server.  The authorization code MUST expire
              shortly after it is issued to mitigate the risk of leaks.  A
              maximum authorization code lifetime of 10 minutes is
              RECOMMENDED.  The client MUST NOT use the authorization code
              more than once.  If an authorization code is used more than
              once, the authorization server MUST deny the request and SHOULD
              attempt to revoke all tokens previously issued based on that
              authorization code.  The authorization code is bound to the
              client identifier and redirection URI.
        state
              REQUIRED if the "state" parameter was present in the client
              authorization request.  The exact value received from the
              client.
        """
        self.code = code
        self.state = state



"""OAuth 2.0 WSGI server middleware providing MyProxy certificates as access tokens
"""
__author__ = "R B Wilkinson"
__date__ = "12/12/11"
__copyright__ = "(C) 2011 Science and Technology Facilities Council"
__license__ = "BSD - see LICENSE file in top-level directory"
__contact__ = "Philip.Kershaw@stfc.ac.uk"
__revision__ = "$Id$"

from abc import ABCMeta, abstractmethod


class AuthenticatorInterface(object):
    """
    Interface for user authentication.
    """
    __metaclass__ = ABCMeta

    def __init__(self, typ):
        """Initialises the authenticator
        @type typ: str
        @param typ: type of authenticator (usually 'client' or 'resource')
        """
        self.typ = typ
        super(AuthenticatorInterface, self).__init__()

    @abstractmethod
    def authenticate(self, params):
        """Authenticates a user.
        @type params: dict
        @param params: request parameters

        @rtype: bool
        @return: True if user authenticated
        """
        return None



"""OAuth 2.0 WSGI server middleware providing MyProxy certificates as access tokens
"""
__author__ = "R B Wilkinson"
__date__ = "12/12/11"
__copyright__ = "(C) 2011 Science and Technology Facilities Council"
__license__ = "BSD - see LICENSE file in top-level directory"
__contact__ = "Philip.Kershaw@stfc.ac.uk"
__revision__ = "$Id$"
from abc import ABCMeta, abstractmethod


class AccessTokenRequest(object):
    """
    OAuth 2.0 access token request object.
    """
    def __init__(self, grant_type, code, redirect_uri):
        """
        grant_type
              REQUIRED.  Value MUST be set to "authorization_code".
        code
              REQUIRED.  The authorization code received from the
              authorization server.
        redirect_uri
              REQUIRED, if the "redirect_uri" parameter was included in the
              authorization request as described in Section 4.1.1, and their
              values MUST be identical.
        """
        self.grant_type = grant_type
        self.code = code
        self.redirect_uri = redirect_uri



class AccessTokenResponseBase:
    """OAuth 2.0 access token response base class.
    """
    __metaclass__ = ABCMeta
    
    def __init__(self, access_token, token_type, expires_in):
        """Set access token, type, expiry.
        """
        self.access_token = access_token
        self.token_type = token_type
        self.expires_in = expires_in

    @abstractmethod
    def get_as_dict(self):
        content_dict = {'access_token': self.access_token,
                        'token_type': self.token_type,
                        'expires_in': self.expires_in}

        return content_dict
    
    
class AuthzCodeGrantAccessTokenResponse(AccessTokenResponseBase):
    """OAuth 2.0 access token response for Authorisation Grant code flow.
    """
    def __init__(self, *arg, **kwarg):
        """Set access token, type, expiry and optionally, a refresh token.
        Refresh token should not be set for the Implicit Grant flow
        """
        self.refresh_token = kwarg.pop('refresh_token', None)
        
        super(AuthzCodeGrantAccessTokenResponse, self).__init__(*arg, **kwarg)

    def get_as_dict(self):
        content_dict = super(AuthzCodeGrantAccessTokenResponse, 
                             self).get_as_dict()
        
        if self.refresh_token:
            content_dict['refresh_token'] = self.refresh_token
            
        return content_dict


class ImplicitGrantAccessTokenResponse(AccessTokenResponseBase):
    """OAuth 2.0 access token response for Implicit Grant code flow.
    """
    def __init__(self, access_token, token_type, expires_in, state, scope=None):
        """Set access token, type, expiry, state and optionally, scope.
        """
        super(ImplicitGrantAccessTokenResponse, self).__init__(access_token, 
                                                               token_type, 
                                                               expires_in)
        self.state = state
        self.scope = scope
        
    def get_as_dict(self):
        content_dict = super(ImplicitGrantAccessTokenResponse, 
                             self).get_as_dict()
                  
        content_dict['state'] = self.state
        if self.scope:
            content_dict['scope'] = self.scope
            
        return content_dict